# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'
use strict;
use vars qw($loaded);

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..1\n"; }
END {print "not ok 1\n" unless $loaded;}
use Data::Squirrel;
use Data::Squirrel::File;
use Data::Squirrel::MySQL;
use Data::Squirrel::DB_File;
use Data::Dumper;
$loaded = 1;
print "ok 1\n";

######################### End of black magic.

# Insert your test code below (better if it prints "ok 13"
# (correspondingly "not ok 13") depending on the success of chunk 13
# of the test code):

my $n=2;

my %tests=(
'MySQL'=>{DataSource=>"dbi:mysql:test", UserName=>"test", Password=>"test", 
          LockDataSource=>"dbi:mysql:test", LockUserName=>"test", LockPassword=>"test", },
'File'=>{Directory=>'test', LockDirectory=>'test'},
'DB_File'=>{FileName=>'test/objects.db', , LockDirectory=>'test'},
);

my @testdata=(
'hello',
[qw(hello world)],
{hello=>'world', biff=>{bibble=>'borz'}, zittle=>[qw(miff moff)]},
time,
);

system "mkdir", "-p", 'test';

foreach my $type (qw(File MySQL DB_File)) {
    print "# $type\n";
    my $key='AF0001';
    my %squirrel;
    tie %squirrel, "Data::Squirrel::$type", $tests{$type};
    unless(tied %squirrel) {
        print "not ok $n\n";
        $n++;
        foreach (@testdata) {
            print "skip $n\n";
            $n++;
        }
        next;
    }
    print "ok $n\n";
    $n++;

    foreach my $in (@testdata) {
        $squirrel{$key}=$in;
        my $out=$squirrel{$key};
        if(not defined $out) {
            print "not ok $n\n";
            $n++;
            print "skip $n\n";
            $n++;
            $key++;
            next;
        }        

        unless(data_eq($in, $out)) {
            warn Dumper $in, $out;
            print 'not ';
        }
        print "ok $n\n";
        $n++;
        $key++;
    }
}

warn "Sleeping 5 seconds...\n";
sleep 5;

my $key='AF0001';
foreach my $type (qw(File MySQL DB_File)) {
    print "# $type\n";

    my %squirrel;
    tie %squirrel, "Data::Squirrel::$type", $tests{$type};
    if(tied %squirrel) {
        tied(%squirrel)->expire(4);  # expire everything under 4 seconds
        print 'not ' if exists $squirrel{$key};
    } 
    else {
        print 'not ';
    }
    print "ok $n\n";
    $n++;
}

# system "rm", "-fR", 'test';     # should be empty, except for AF0001's lock

sub data_eq
{
    my($in, $out)=@_;
    my $r=ref $in;
#    return 0 unless $r eq ref $out;
    unless($r) {
        return 1 if $in eq $out;
        return 0;
    } elsif($r eq 'ARRAY') {
        foreach my $q (0..$#$in) {
            return 0 unless data_eq($in->[$q], $out->[$q]);
        }
    } elsif($r eq 'HASH') {
        foreach my $k (keys(%$in), keys(%$out)) {
            return 0 unless exists $in->{$k} and exists $out->{$k};
            return 0 unless data_eq($in->{$k}, $out->{$k});
        }
    }
    return 1;
}
